import fetch from 'node-fetch'
import { config } from './config.js'
import { getRedisKey, hash } from './util.js'
import { logger } from './logger.js'
import { client, pubClient } from './redis.js'

let latestVersion

export const fetchLatestVersion = async () => {
  await config.load()
  const infos = await Promise.all(config.urls.map(async baseUrl => {
    try {
      const response = await fetch(new URL('meta.json', baseUrl))
      if (!response.ok) throw new Error()
      const meta = await response.json()
      const version = meta.commitSha || meta.buildDate || meta.version
      if (!version) throw new Error()
      return version
    } catch (err) {
      logger.debug(`UI container at ${baseUrl} does not have meta.json. Fall back to version hash based on manifest.`)
    }
    try {
      const response = await fetch(new URL('manifest.json', baseUrl))
      if (!response.ok) throw new Error()
      const manifest = await response.json()
      return hash(manifest)
    } catch (err) {
      logger.error(`Cannot fetch manifest from ${baseUrl}. Version info will not be correct.`)
    }
  }))
  return hash(infos)
}

export async function getLatestVersion () {
  if (latestVersion) return latestVersion

  const version = await client.get(getRedisKey({ name: 'latestVersion' }))
  if (version) return (latestVersion = version)

  const newVersion = await fetchLatestVersion()
  pubClient.publish(getRedisKey({ name: 'updateLatestVersion' }), newVersion)
  await client.set(getRedisKey({ name: 'latestVersion' }), newVersion)
  return (latestVersion = newVersion)
}

export function registerLatestVersionListener (client) {
  const key = getRedisKey({ name: 'updateLatestVersion' })
  client.subscribe(key, (errs, count) => logger.info(`Subscribed to ${key}.`))
  client.on('message', (channel, message) => {
    if (channel === key) latestVersion = message
  })
}

export async function updateVersionProcessor (job) {
  const [storedVersion, fetchedVersion] = await Promise.all([
    getLatestVersion(),
    fetchLatestVersion()
  ])
  if (storedVersion === fetchedVersion) return fetchedVersion
  pubClient.publish(getRedisKey({ name: 'updateLatestVersion' }), fetchedVersion)
  await client.set(getRedisKey({ name: 'latestVersion' }), fetchedVersion)
  return fetchedVersion
}
